
# q-expansion of newform 64.16.a.e, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 64
weight = 16
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-276], [132210], [3585736], [47801700], [-247784966], [-2127682062], [-1074862756], [-24982896168], [165099671946], [-100736332256], [-42490420334], [-1388779245414], [-1168783477180], [1645655322672], [4469627500578], [-28794808426572], [-15719941145942], [61627103890604], [66780412989192], [-57749646345094], [-198700138788272], [-113345193514212], [-48230883277974], [95121696327074], [-1158329724393006], [-1617570441046232], [370312450248708], [2171515869935962], [152160176525202], [-4076367364175744], [-11169530059396500], [-6364771585687494], [-16499119385196316], [3302198652971394], [-4314728664964520], [17476851519020554], [49516867325427020], [23391486044757864], [-53317922235694182], [44172682189352892], [85233647131983010], [164872064399355072], [167090246544734018], [-106538461090601166], [197125441188118408], [-203915960198113636], [-12469928052012320], [590160091413670668], [-189414533609476526], [125115861525478362], [-262040610860473104], [1327648819049234], [-1474449543576607500], [-224341178526527742], [-210162294444083256], [1114500173420661498], [-1396171124953518320], [725940419695218178], [-1388541569956642134], [-4171872242953885228], [5976983156772742290], [-7162912415334896836], [-5553871303446233352], [-5231279834991537526], [-6512270252717950038], [11432985791502321188], [15416748848891155634], [-4177594305826993260], [9178639775779366090], [-16526930223238575966], [-335788689752836056], [-10076565263160730256], [-9004624590845522078], [28243153206873925748], [12153935248705763712], [41807468420834021874], [-13701244936450621574], [45329933403366214770], [-15315115800384844246], [-54449443139467574196], [23339975455612354834], [41454766012339280304], [-54392369095643572654], [-88071841251292266632], [77824505639635360308], [127118388147098224194], [-6200155592490080518], [-16506996818829340230], [-128678440944883518512], [-111384458586246095460], [74344229236553243616], [91032475539231754408], [-154954173613498271100], [79800448316928609404], [135801475993297760184], [19898758681648878186], [-213097602319942635846], [-223159002982547172508], [-37386105065371350134], [435348293854911627212], [-385861891444919426790], [154528358178248311356], [-621278664049895501430], [189079235869636591028], [330668143216853590466], [995916398171479264548], [17428284208842519474], [-1082466445210410753000], [1362154867850260758890], [840943376287190238304], [1369096420901288084434], [-611863438580349322662], [848529473106169970180], [246175047210919473208], [-797105466450451717758], [1974367518740000159852], [-695727204324705605304], [1723851506869957138554], [-1187110670702874497316], [533810238855770303362], [2171882633146728171362], [665673077960818868754], [580477444567971846084], [703252861301247663548], [2358954561199455199530], [-2254534609117499739854], [-1606609759905563548464], [1867813319309946537112], [3050324095687572664906], [4190467082773103859980], [-4434702041547028190808], [2920034709893760485104], [-726555520007266562846], [4279614708302685409482], [-8117097764105575905022], [250509263006934955890], [-1418370418102623167908], [-5733118912442429217654], [-1911377279220076442214], [-5228150623253872207036], [-2676235721451865918686], [7878244887542762406136], [-5829477341635005373260], [8013686962781437306282], [8324419372631820112392], [15279813597709079862850], [10156568901857840184426], [13837400629646255954324], [-14475703995404724505248], [4826743109324227929178], [-11993859882466609746798], [-7578069849744312148228], [-4603620254133854957256], [-12261746175958512198172], [-26756048625152694679152], [-16382947135123278699176], [30447741298066866394722], [17720313957052351545626], [26334475683718766626458], [-6918721196255271222084], [-22501303390065378930678], [19616495810323014148744], [-1139026181912610261084], [6235396418621184042546], [-51385737774816067205736], [3044761972771096813024], [36043019547216085913938], [9719431263138424389650], [41407599673686964532706], [-66669168034992951965196], [-39786002005303923655574], [56347304207236600519368], [44475711136346152386746], [-28932008733293157514736], [43060302453452685983658], [-45285539518375787426092], [-12994711862308920346734], [18685672860406728258280], [10466343951671334503194], [87923725557177419458624], [39934188016815256303788], [12376510974143330324146], [-104906744711498831378310], [-126464551044045744771504], [53888979673188956753730], [-54543964832023556962358], [-51317256231925957563508], [-57269497567722513144742], [-147040602163091992662912], [67155713444099820630914], [-162326240031172078672668], [43805681078873472855260], [-164085272351464807175670], [-115835103079193010857652], [-169707319013246211805158], [62100180115302786111314], [128126627113385172755242], [59967777525331918919490], [184566926821426204615560], [-54903063901897784048454], [-229185790639392005798192], [124314296152458316686274], [2522123210855907142946], [-174543164601933289742460], [56200759510892131640682], [277597925465060443002112], [-2535054308961216663060], [100736706206492211613626], [-18060913053884353009564], [-417065299215557868564238], [-56364141613700224607742], [103094749310134613796568], [427640859592597736433108], [-134361574537381729250328], [285247167635863040494490], [354047539593681833297200], [-160711945003486946636622], [-601918105766753241749736], [-352630916297458934572854], [-176077404248631441015086], [359910110874444265976632], [-703788733443874167320958], [-366302241133432388187248], [-365111696949115056970788], [-647327471864997997650302], [-124942572217674439659990], [261098493707587916368800], [-222824180238300929366936], [-23484552363364375352124], [341163705951792805791898], [65982231771393472910012], [-458128251987434174324288], [198288533870290599417594], [-53547637287483720530524], [23423547735559198046160], [10467025076399984698418], [-308277535397223708953406], [-890908294207285069333740], [98221820265143326313640], [-718859425791864634611588], [-1082037140593123435989004], [-1103918958492999198008120], [907197158778851378692090], [-456748824949975951154190], [-908837591762638118230056], [1517052273443830187783200], [1788792794805111639423948], [1083524598028609911914564], [1757977113864647663236656], [-121494476935206024324950], [-25035993325250373214782], [-97541069143071052802808], [1811068624595109807404666], [1061702774158231039101498], [435216235138934985463068], [-1889802878838520302034622], [474878932813097863728788], [-2374759456828497721069614], [-38230913236462907191990], [821576146008316458618496], [-909925590863892258560916], [805080380793814806298354], [-3227337553914215631221750], [2682171751234111120089954], [736049948219327260890956], [3115318232926688975896986], [343511892786118642733322], [1939988933599534080519476], [-232093393897514266893006], [2984942022524977650529978], [-2790677252442615261048676], [-3669036241210980504810262], [-1140209623106269254430496], [1271408605756952529006866], [-3579624751045614463257416], [445521663692664982503156], [-2302097966621330778153110], [-626461083671615576289862], [-909562976129157035686308], [-989491240373745722637024], [-831597433832630361016856], [-2451088587141209234250504], [5323429559455394455108018], [194053301096996370931748], [799714946983896086692176], [3870371535293734875712178], [-2769567940158858694073790], [-1699425404386605924710120], [-3306708478445549770759518], [2233046226627676469717082], [713568298712135973346044], [-4116652848838410925076406], [-5977614237744623920838172], [6688834808567524743169402], [-4216907530828562957800182], [-2615204129143941427262048], [7555101001445910199548834], [-232900384192351353428940], [-6896439100680296454878932], [6516972563437589406515450], [5238666516105846348218298], [7750909165190145632565712], [5036786128483648402364316], [-4676528842259346354632428], [6929103442626995414955554], [8939751712177536667038852], [945229547389555378230490], [-3372008563441779962817096], [-6761867351206729528970126], [5378331971194927626225168], [910497901987935623120130], [11771268815914323016972770], [-3486348245206493146781236], [-1779632743568492296079640], [2484242252063002672417946], [-14689059463662902269338564], [-10663816684942332175061952], [-16276534825357330284999742], [7608404363806875801177522], [-9958408533626255827754212], [2977676972489103486531434], [1108000526505394281513930], [10494271022282363371465312], [12023639781292936145387610], [5658273040460742804099986], [-20873228632327900968971668], [-3033461269636771335422380], [-1138006767642795801682272], [-864820174345641672155118], [-3857448510328639524021350], [3868604923784636678041386], [-15559307892451664653970240], [6260839760508103631872044], [-14255031201747744195233116], [19132636553490101142151188], [15637543968252608829311050], [-18363641024052217772779230], [-10887531412999650512239654], [10404750258266268104805616], [-31830481349561366775885086], [-17344906926326549514024246], [-30224931106085476298527374], [1159221100510570348609480], [-32419088604514444890404214], [-7475361186177022762438452], [-19090787883093081921381230], [8697528676584609702708548], [14039173731665456286943536], [-25644363706044031459413726], [-6725652681410167517172820], [-7354444613640577530687622], [15738732189992385702636858], [26428394758801770950476624], [34525685315720483961809986], [10508382471977535371594346], [19898111030903487109675872], [-1516297118841942021366012], [-6624369606102833775959406], [-6624295007615511365773512], [-7348987600146109025785358], [-16604547031271744663602374], [-30171851964374164436848752], [4449391037254255853720916], [-33415397107712140886537140], [-50413184403209305405519078], [29157671047451203031008410], [43851905993172812839520392], [14673395864810319719757290], [9677262469044407111010828], [-39621007149259389976706428], [24539871739058262406938096], [26209655310977787749790690], [-18733082666042804519831624], [-52034358266211649066630550], [-69447250320723657069872292], [-71340100015247110700386272], [49537914285088389522455522], [-10492632240490324343421486], [-59026130332745790710896246], [-40265125988556877705106262], [-28445676045326017054716294], [-75239377927046559603110888], [-50947197826450423218298974], [10309397132756116178736524], [-18519626941103252851564248], [49269068530715524386510448], [38630115723341448334180450], [2982549645349442217969524], [4950753614621113602462336], [-49883496092486873400062590], [25630435499720450649559794], [75777467183195860927569636], [-5281156206919037011089700], [-37179275704675996013753256], [-29429199691835001582549718], [39161046100222391413629088], [-79375387992661044901767654], [60796521395007513680678852], [-25825578803635918165221726], [-30986080557380438202420950], [106682538242111937718504770], [10271073391904191531969744], [-110817497638692274399423254], [-23190212549654055259604910], [-41204046578007314740997720], [17898602966765283870179290], [-27483883042821502905849582], [90188637104254879456664444], [85173097661726364227578860], [97984162039183666058841842], [-28332263568902402560452606], [17749248969897704376811476], [-47243091530016179635650868], [105010742978540775996193178], [116777027010314855313256986], [-162590452183464959156969664], [112192019701284998227409416], [-103374842555000259275135310], [72873050155348576093678872], [79536529313330412661012170], [-39926046707470692699505454], [-33638565933336070943668368], [32541714790945359785127540], [99121789634394405948699962], [-168737105231310059298238086], [82662423357800478768532956], [109745424796541434194192426], [65015905234039976005284692], [-104305648010908102995097224]]
